/**
 * Example of handling a submit event using a form tag in HTML.
 * Note that the "submit" event is the event that fires whenever
 * a button inside a <form> is clicked. There are different types
 * of forms you can write in HTML, but they have often been misused.
 * You can use them though when:
 * - You are using input elements that semantically are grouped in a form
 *   used to submit data to a server.
 * - You want to use HTML5 validation features to check input elements
 *   before the submit event is fired.
 * - You are keeping user experience in mind, letting a user know if they
 *   have invalid input (e.g. using JS to display helpful error messages)
 *   and when they have successfully sent their data to the server.
 * - Be cognizant of security when verifying sensitive data! Refer to MDN's
 *   web security overview or OWASP for best practices.
 * - Ask on Piazza/OH if you want to learn more about when/how to use these!
 */
(function() {
  "use strict";
  const URL = "submitter.php";

  window.addEventListener("load", initialize);

  /**
   * Sets up the listeners for submitting to our simple form example.
   */
  function initialize() {
    qs("form").addEventListener("submit", function(e) {
      // if we've gotten in here, all html5 validation checks have passed
      e.preventDefault(); // submit action on forms by default redirect page, so prevent that behavior
      submitRequest();
    });
  }

  /**
   * Submits a request using the form values provided by a user
   * to submitter.php. Note that this function won't be called until
   * the HTML5 validation checks have been passed, due to the way
   * form submit events are handled in JavaScript!
   */
  function submitRequest() {
    // Ideally, we'd also use JS to validate before submitting to server
    let url = URL;

    // Can use form DOM element as parameter to FormData!
    let form = qs("form");
    let params = new FormData(form);
    fetch(url, { method : "POST", body : params })
      .then(checkStatus)
      .then(showResponse)
      .catch(console.log);
  }

  /**
   * Displays the text response from the server (submitter.php).
   * @param {string} responseText
   */
  function showResponse(responseText) {
    $("result").innerText = responseText;
  }

  /* ------------------------------ Helper Functions  ------------------------------ */

  /**
   * Returns the element that has the ID attribute with the specified value.
   * @param {string} id - element ID
   * @returns {object} DOM object associated with id.
   */
  function $(id) {
    return document.getElementById(id);
  }

  /**
   * Returns the first element that matches the given CSS selector.
   * @param {string} query - CSS query selector.
   * @returns {object} The first DOM object matching the query.
   */
  function qs(query) {
    return document.querySelector(query);
  }

  /**
   * Helper function to return the response's result text if successful, otherwise
   * returns the rejected Promise result with an error status and corresponding text
   * @param {object} response - response to check for success/error
   * @return {object} - valid result text if response was successful, otherwise rejected
   *                    Promise result
   */
  function checkStatus(response) { 
    if (response.status >= 200 && response.status < 300 || response.status == 0) {  
      return response.text();
    } else {  
      return Promise.reject(new Error(response.status + ": " + response.statusText)); 
    }
  }
})();

